@@ -1,5 +1,5 @@
 // rTorrent - BitTorrent client
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -46,13 +46,16 @@
 #include "curl_socket.h"
 #include "curl_stack.h"
 
+namespace std { using namespace tr1; }
+
 namespace core {
 
 CurlStack::CurlStack() :
   m_handle((void*)curl_multi_init()),
   m_active(0),
   m_maxActive(32),
-  m_ssl_verify_peer(true) {
+  m_ssl_verify_peer(true),
+  m_dns_timeout(60) {
 
   m_taskTimeout.set_slot(rak::mem_fn(this, &CurlStack::receive_timeout));
 
@@ -130,9 +133,9 @@ CurlStack::transfer_done(void* handle, const char* msg) {
     throw torrent::internal_error("Could not find CurlGet with the right easy_handle.");
 
   if (msg == NULL)
-    (*itr)->signal_done().emit();
+    (*itr)->trigger_done();
   else
-    (*itr)->signal_failed().emit(msg);
+    (*itr)->trigger_failed(msg);
 }
 
 void
@@ -165,8 +168,8 @@ CurlStack::add_get(CurlGet* get) {
   if (!m_httpCaCert.empty())
     curl_easy_setopt(get->handle(), CURLOPT_CAINFO, m_httpCaCert.c_str());
 
-  if (!m_ssl_verify_peer)
-    curl_easy_setopt(get->handle(), CURLOPT_SSL_VERIFYPEER, 0); 
+  curl_easy_setopt(get->handle(), CURLOPT_SSL_VERIFYPEER, (long)m_ssl_verify_peer); 
+  curl_easy_setopt(get->handle(), CURLOPT_DNS_CACHE_TIMEOUT, m_dns_timeout);
 
   base_type::push_back(get);
 
