@@ -1,5 +1,5 @@
 // rTorrent - BitTorrent client
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -95,7 +95,7 @@ public:
 
   void                set_attr(unsigned int x, unsigned int y, unsigned int n, int attr, int color) { mvwchgat(m_window, y, x, n, attr, color, NULL); }
 
-  void                set_default_attributes(int attr)                            { wattrset(m_window, attr); }
+  void                set_default_attributes(int attr)                            { (void)wattrset(m_window, attr); }
 
   // Initialize stdscr.
   static void         initialize();
