@@ -1,5 +1,5 @@
 // rTorrent - BitTorrent client
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@
 #include "config.h"
 
 #include <torrent/rate.h>
+#include <torrent/tracker_controller.h>
 #include <torrent/tracker_list.h>
 #include <torrent/peer/connection_list.h>
 #include <torrent/peer/peer_list.h>
@@ -68,16 +69,21 @@ WindowDownloadStatusbar::redraw() {
   position = print_download_info(buffer, last, m_download);
   m_canvas->print(0, 0, "%s", buffer);
 
-  position = buffer + std::min<ptrdiff_t>(std::max(snprintf(buffer, last - buffer, "Peers: %i(%i) Min/Max: %i/%i Uploads: %i U/I/C/A: %i/%i/%i/%i Failed: %i",
+  position = buffer + std::min<ptrdiff_t>(std::max(snprintf(buffer, last - buffer, "Peers: %i(%i) Min/Max: %i/%i Slots: U:%i/%i D:%i/%i U/I/C/A: %i/%i/%i/%i Unchoked: %u/%u Failed: %i",
                                         (int)m_download->download()->connection_list()->size(),
                                         (int)m_download->download()->peer_list()->available_list_size(),
                                         (int)m_download->download()->connection_list()->min_size(),
                                         (int)m_download->download()->connection_list()->max_size(),
+                                        (int)m_download->download()->uploads_min(),
                                         (int)m_download->download()->uploads_max(),
+                                        (int)m_download->download()->downloads_min(),
+                                        (int)m_download->download()->downloads_max(),
                                         (int)m_download->download()->peers_currently_unchoked(),
                                         (int)m_download->download()->peers_currently_interested(),
                                         (int)m_download->download()->peers_complete(),
                                         (int)m_download->download()->peers_accounted(),
+                                        (int)m_download->info()->upload_unchoked(),
+                                        (int)m_download->info()->download_unchoked(),
                                         (int)m_download->chunks_failed()),
                                0), last - buffer);
 
@@ -86,7 +92,7 @@ WindowDownloadStatusbar::redraw() {
   position = print_download_status(buffer, last, m_download);
   m_canvas->print(0, 2, "[%c:%i] %s",
                   m_download->tracker_list()->has_active() ? 'C' : ' ',
-                  (int)(m_download->download()->tracker_list()->time_next_connection()),
+                  (int)(m_download->download()->tracker_controller()->seconds_to_next_timeout()),
                   buffer);
 }
 
