@@ -1,5 +1,5 @@
 // rTorrent - BitTorrent client
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@
 #include <sys/socket.h>
 #include <torrent/exceptions.h>
 #include <torrent/poll.h>
+#include <torrent/utils/log.h>
 
 #include "utils/socket_fd.h"
 
@@ -176,12 +177,16 @@ SCgiTask::event_read() {
   worker_thread->poll()->insert_write(this);
 
   if (m_parent->log_fd() >= 0) {
+    int __UNUSED result;
+
     // Clean up logging, this is just plain ugly...
     //    write(m_logFd, "\n---\n", sizeof("\n---\n"));
-    write(m_parent->log_fd(), m_buffer, m_bufferSize);
-    write(m_parent->log_fd(), "\n---\n", sizeof("\n---\n"));
+    result = write(m_parent->log_fd(), m_buffer, m_bufferSize);
+    result = write(m_parent->log_fd(), "\n---\n", sizeof("\n---\n"));
   }
 
+  lt_log_print(torrent::LOG_RPC_DEBUG, "---\n%*s\n---", m_bufferSize - std::distance(m_buffer, m_body), m_body);
+
   // Close if the call failed, else stay open to write back data.
   if (!m_parent->receive_call(this, m_body, m_bufferSize - std::distance(m_buffer, m_body)))
     close();
@@ -234,12 +239,15 @@ SCgiTask::receive_write(const char* buffer, uint32_t length) {
   std::memcpy(m_buffer + headerSize, buffer, length);
 
   if (m_parent->log_fd() >= 0) {
+    int __UNUSED result;
     // Clean up logging, this is just plain ugly...
     //    write(m_logFd, "\n---\n", sizeof("\n---\n"));
-    write(m_parent->log_fd(), m_buffer, m_bufferSize);
-    write(m_parent->log_fd(), "\n---\n", sizeof("\n---\n"));
+    result = write(m_parent->log_fd(), m_buffer, m_bufferSize);
+    result = write(m_parent->log_fd(), "\n---\n", sizeof("\n---\n"));
   }
 
+  lt_log_print(torrent::LOG_RPC_DEBUG, "---\n%*s\n---", m_bufferSize, m_buffer);
+
   event_write();
   return true;
 }
